//FILE: libgomd.cpp
//PURPOSE: implement a simple class library loaded dynamically at runtime by a program
//AUTHOR: Gian Paolo Ghilardi, a.k.a. rejected || jp
//MAIL: (jp80@tin.it)

//TO COMPILE: g++ libgomd.cpp  -o libgomd.so -shared

#include "lgomd.hpp"
#include "libgomdconsts.hpp"
#include <iostream>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>

int readFile(const char* fileName, int nodeNumber, int *value);


using namespace std;

/***************************************************************************************
 * LIBGOMD CLASS DEFINITION                                                            *
 * This is the definition of the exported class.                                       *
 * Contains some functions visible by the client program                               *
 ***************************************************************************************/
class libgomd : public lgomd //extends the lgmod interface
{
	public:
		void getLoad( int nodeId );
};

//getLoad method definition (was set as "virtual" in lgomd)
void libgomd::getLoad( int nodeId )
{
	int* value;
	readFile("load", nodeId , value);
	cout << "- Load of node #" << nodeId << " is " << *value << endl;
}
/***************************************************************************************/



/***************************************************************************************
 * EXPORTED LIBRARY FUNCTIONS                                                          *
 * Functions called by the dynamic library loader                                      *
 * NOTE: these two functions are declared as 'extern "C"' so they can be               *
 * called by Linux's standard library loading functions (dlopen,dlsym,dlclose,dlerror) *
 ***************************************************************************************/
extern "C" lgomd* create()
{
	//HERE IS THE TRICK: we return a class!!! :)
	return new libgomd; //returns a new instance of the libgomd class to the caller
}

extern "C" void destroy(lgomd* instance)
{
	delete instance; //kills a valid libgomd instance
}
/***************************************************************************************



/***************************************************************************************
 * I/O FUNCTIONS                                                                       *
 * Functions used to load oM values from the file of the nodes                         *
 * (These files are in /proc/hpc/nodes/NODE_ID/)                                       *
 ************************************************************************************* */
int readFile(const char* fileName, int nodeNumber, int *value)
{
    FILE *fp;
    char targetFile[256];
    char *targetFilePtr = targetFile;
    char c;

    sprintf(targetFilePtr, "%s/%d/%s", PROC_HPC_NODES_DIR, nodeNumber, fileName);

    if ((fp = fopen(targetFilePtr,"r")) == NULL)
    {
	return LGOMD_ERR_READ_FAILED;
    }
    else
    {
	fscanf(fp, "%i", value);
	fclose(fp);
	return (LGOMD_SUCCESS);
    }
}
 /***************************************************************************************/
